@@ -10,6 +10,18 @@ import collections
 from FModel.models import Clients
 BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 savefile=os.path.join(BASE_DIR, 'static', 'modelfile','clients.tsv')
+
+import random
+def randomcolor():
+    colors=[]
+    for _ in range(50):
+        colorArr = ['1','2','3','4','5','6','7','8','9','A','B','C','D','E','F']
+        color = ""
+        for i in range(6):
+            color += colorArr[random.randint(0,14)]
+        colors.append("#"+color)
+    return colors
+
 #
 alive_client={}
 def setsession(key):
@@ -44,18 +56,18 @@ def setClientStatus(key,status):
             client.save()
             result=1
     return result
-def getClientId(cname,key=None):
+def getClientId(cname,modeltype,key=None):
     context={}
     context['result']=0
-    if not key.strip():
+    if not key or (key and not key.strip()):
         count=Clients.objects.count()
-        key=getMD5('%s%s'%(count,cname))
-        ncl=Clients.objects.create(CName=cname,Key=key,Status=0)
+        key=getMD5('%s%s%s'%(count,cname,modeltype))
+        ncl=Clients.objects.create(ModelType=modeltype,CName=cname,Key=key,Status=0)
         context['clientid']=ncl.CID
         context['key']=key 
         print(context)
-    elif key.strip() and not Clients.objects.filter(Key=key).first():
-        ncl=Clients.objects.create(CName=cname,Key=key,Status=0)
+    elif key and key.strip() and not Clients.objects.filter(Key=key).first():
+        ncl=Clients.objects.create(ModelType=modeltype,CName=cname,Key=key,Status=0)
         context['clientid']=ncl.CID
         context['key']=key 
     elif Clients.objects.filter(Key=key,Status=1).first():
@@ -66,11 +78,14 @@ def average_weights(w):
     Returns the average of the weights.
     """
     w_avg = copy.deepcopy(w[0])
-    for key in w_avg.keys():
-        for i in range(1, len(w)):
-            w_avg[key]=w_avg[key].cpu()
-            w_avg[key] +=w[i][key].cpu()
-        w_avg[key] = torch.div(w_avg[key], len(w))
+    try:
+        for key in w_avg.keys():
+            for i in range(1, len(w)):
+                w_avg[key]=w_avg[key].cpu()
+                w_avg[key] +=w[i][key].cpu()
+            w_avg[key] = torch.div(w_avg[key], len(w))
+    except Exception as ex:
+        print('please check model type or clean catch',ex)
     return w_avg
 
 def average_weights_enc(w):
@@ -79,24 +94,29 @@ def average_weights_enc(w):
     """
     new_w = collections.OrderedDict()
     w_avg = copy.deepcopy(w[0])
-    for key in w_avg.keys():
-        for i in range(1, len(w)):
-            w_avg[key]=w_avg[key].cpu()
-            w_avg[key] +=w[i][key].cpu()
-        # w_avg[key] = torch.div(w_avg[key].get_plain_text(), len(w))
-        new_w[key.replace('.data','')]=torch.div(w_avg[key].get_plain_text(), len(w))
+    try:
+        for key in w_avg.keys():
+            for i in range(1, len(w)):
+                w_avg[key]=w_avg[key].cpu()
+                w_avg[key] +=w[i][key].cpu()
+            # w_avg[key] = torch.div(w_avg[key].get_plain_text(), len(w))
+            new_w[key.replace('.data','')]=torch.div(w_avg[key].get_plain_text(), len(w))
+    except Exception as ex:
+        print('please check model type or clean catch',ex)
     return new_w
 
 def check_fed_model(save_dir,keys,type,test_acc=None,test_loss=None,EpochCount=None):
-    print('check_fed_model',save_dir,keys)
+    print('check_fed_model',save_dir,keys,type)
     # addtasklog(save_dir,clientnumber,taskname,'test_acc',test_acc,'test_loss',test_loss,_clientid)
     result=1
     cmodels=[]
     try:
         for key in keys:
-            model=torch.load(os.path.join(save_dir,key))
-            cmodels.append(model)
-        result=3
+            if os.path.exists(os.path.join(save_dir,key)):
+                model=torch.load(os.path.join(save_dir,key))
+                cmodels.append(model)
+        if(len(keys)==len(cmodels)):
+            result=3
     except Exception as es:
         print(es)
     if result==3:
@@ -107,9 +127,10 @@ def check_fed_model(save_dir,keys,type,test_acc=None,test_loss=None,EpochCount=N
         if avg_model:
             for key in keys:
                 try:
-                    model=torch.load(os.path.join(save_dir,key))
-                    torch.save(avg_model,os.path.join(save_dir,'%s_model'%(key)))
-                    os.remove(os.path.join(save_dir,key))
+                    if os.path.exists(os.path.join(save_dir,key)):
+                        model=torch.load(os.path.join(save_dir,key))
+                        torch.save(avg_model,os.path.join(save_dir,'%s_model'%(key)))
+                        os.remove(os.path.join(save_dir,key))
                 except:
                     pass
     return result
