@@ -9,6 +9,7 @@ from . import Util
 from FModel.models import Clients
 BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 save_dir=os.path.join(BASE_DIR, 'static', 'modelfile')
+colors=Util.randomcolor()
 def log_in(func):
     def wrapper(request,*args,**kwargs):
         return  func(request,*args, **kwargs)
@@ -16,17 +17,22 @@ def log_in(func):
 @log_in
 def index(request,context={}): 
     clients=Clients.objects.all()
-    print('clients',len(clients))
+    print('client count',len(clients))
     clientsList=[]
     for cl in clients:
         item={}
         item['id']=cl.CID
         item['name']=cl.CName
         item['key']=cl.Key
+        item['ModelType']=cl.ModelType
         item['loss']='%.4f'%float(cl.Loss) if cl.Loss.strip() and float(cl.Loss)>0 else ''
         item['status']=cl.Status
         item['checked']='checked' if int(cl.Status)==1 else ''
         item['color']='green' if Util.getsession(cl.Key)>0 else 'blue'
+        item['bgcolor']='background:%s'%(colors[int(cl.ModelType)])
+        check=1 if os.path.exists(os.path.join(save_dir,'%s_modelb'%(cl.Key))) else 0
+        item['check_model_file']=check
+        item['down']='/download?key=%s'%cl.Key if check==1 else '#'
         clientsList.append(item)
     context['clients']=clientsList
     context['clients_count']=len(context['clients'])
@@ -37,8 +43,9 @@ def client_login(request,context={}):
     if request.method == 'POST':
         cname=request.POST.get('clientName')
         key=request.POST.get('key')
+        modeltype=int(request.POST.get('modeltype'))
         print(cname,context)
-        context=Util.getClientId(cname,key)
+        context=Util.getClientId(cname,modeltype,key)
         context['logggin']=1
         Util.setsession(key)
         # request.session.modified = True
@@ -51,18 +58,21 @@ def send_model(request):
         test_acc=request.POST.get('acc')
         test_loss=request.POST.get('loss')
         type=request.POST.get('type')
+        modeltype=int(request.POST.get('modeltype'))
         modelfile=request.FILES.get('file')
         with  open(os.path.join(save_dir,key), 'wb') as f:
             for chunk in modelfile.chunks():
                 f.write(chunk)
         ncl=Clients.objects.filter(Key=key).first()
         if ncl and test_loss:
-            if int(test_loss)>0:
+            if float(test_loss)>0:
                 ncl.Loss=test_loss
                 ncl.save()
-        keys=[cl.Key for cl in Clients.objects.filter(Status=1)]
+        keys=[cl.Key for cl in Clients.objects.filter(ModelType=modeltype,Status=1)]
+        # print(len(keys))
         result=Util.check_fed_model(save_dir,keys,type,test_acc,test_loss,10)
         context['result']=result
+        Util.setsession(key)
         print('send_model',context)
     return HttpResponse(json.dumps(context),content_type="application/json")
 @csrf_exempt
@@ -70,7 +80,9 @@ def get_status(request):
     context= {}
     if request.method == 'POST':
         key=request.POST.get('key')
-        keys=[cl.Key for cl in Clients.objects.filter(Status=1)]
+        type=request.POST.get('type')
+        modeltype=int(request.POST.get('modeltype'))
+        keys=[cl.Key for cl in Clients.objects.filter(ModelType=modeltype,Status=1)]
         context['modelurl']=''
         context['clientstatus']=Util.check_client(save_dir,key)
         context['result']=Util.check_client_all(save_dir,keys)
@@ -79,11 +91,12 @@ def get_status(request):
             if os.path.exists(os.path.join(save_dir,'%s_model'%(key))):
                 os.rename(os.path.join(save_dir,'%s_model'%(key)),os.path.join(save_dir,'%s_modelb'%(key)))
                 context['modelurl']=os.path.join(save_dir,'%s_modelb'%(key)).replace(BASE_DIR,'')
-            else:
-                Util.check_fed_model(save_dir,keys,0,0,10)
+            # else:
+            #     Util.check_fed_model(save_dir,keys,type,0,10)
         if int(context['result'])==3 and int(context['clientstatus'])==1:
-            Util.check_fed_model(save_dir,keys,0,0,10)
+            Util.check_fed_model(save_dir,keys,type,0,10)
         print(context)
+        Util.setsession(key)
         # Util.addtasklog(ut.save_dir,'get_status',taskid,clientid,'result',context['result'],context['modelurl'],context['clienttrain'])
     return HttpResponse(json.dumps(context),content_type="application/json") 
 
@@ -96,7 +109,20 @@ def changeStatus(request):
         context['result']=Util.setClientStatus(key,status)
     return HttpResponse(json.dumps(context),content_type="application/json") 
 
-
+from django.http import FileResponse  
+def download(request):
+    context={}
+    if request.method == 'GET':
+        key=request.GET.get('key')
+        client=Clients.objects.filter(Key=key).first()
+        if client:
+            file=open(os.path.join(save_dir,'%s_modelb'%(key)),'rb')
+            response =FileResponse(file)
+            response['Content-Type']='application/octet-stream'
+            response['Content-Disposition']='attachment;filename="%s.model"'%(client.CName)
+            return response
+    return HttpResponse(json.dumps(context),content_type="application/json") 
+ 
 @csrf_exempt
 def deleteclient(request):
     context= {}
